@@ -1,11 +1,12 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
-from SettingsWidgets import *
 import gi
 gi.require_version('CinnamonDesktop', '3.0')
 gi.require_version('UPowerGlib', '1.0')
 from gi.repository import CinnamonDesktop, Gdk, UPowerGlib
 
+from GSettingsWidgets import *
+
 POWER_BUTTON_OPTIONS = [
     ("blank", _("Lock the screen")),
     ("suspend", _("Suspend")),
@@ -39,6 +40,8 @@ SLEEP_DELAY_OPTIONS = [
     (1800, _("30 minutes")),
     (2700, _("45 minutes")),
     (3600, _("1 hour")),
+    (7200, _("2 hours")),
+    (10800, _("3 hours")),
     (0, _("Never"))
 ]
 
@@ -82,6 +85,7 @@ def get_timestring(time_seconds):
 
 
 CSD_SCHEMA = "org.cinnamon.settings-daemon.plugins.power"
+CSM_SCHEMA = "org.cinnamon.SessionManager"
 
 class Module:
     name = "power"
@@ -96,18 +100,18 @@ class Module:
         if self.loaded:
             # self.loaded = False
             return
-        print "Loading Power module"
+        print("Loading Power module")
 
-        self.up_client = UPowerGlib.Client()
+        self.up_client = UPowerGlib.Client.new()
 
         self.csd_power_proxy = Gio.DBusProxy.new_sync(
-                Gio.bus_get_sync(Gio.BusType.SESSION, None),
-                Gio.DBusProxyFlags.NONE,
-                None,
-                "org.cinnamon.SettingsDaemon",
-                "/org/cinnamon/SettingsDaemon/Power",
-                "org.cinnamon.SettingsDaemon.Power",
-                None)
+            Gio.bus_get_sync(Gio.BusType.SESSION, None),
+            Gio.DBusProxyFlags.NONE,
+            None,
+            "org.cinnamon.SettingsDaemon.Power",
+            "/org/cinnamon/SettingsDaemon/Power",
+            "org.cinnamon.SettingsDaemon.Power",
+            None)
 
         self.settings = Gio.Settings.new("org.cinnamon")
 
@@ -124,7 +128,7 @@ class Module:
 
         section = power_page.add_section(_("Power Options"))
 
-        lid_options, button_power_options, critical_options = get_available_options(self.up_client)
+        lid_options, button_power_options, critical_options, can_suspend, can_hybrid_sleep = get_available_options(self.up_client)
 
         size_group = Gtk.SizeGroup(mode=Gtk.SizeGroupMode.HORIZONTAL)
 
@@ -149,9 +153,9 @@ class Module:
                 section.add_row(GSettings2ComboBox(_("When the lid is closed"), CSD_SCHEMA, "lid-close-ac-action", "lid-close-battery-action", lid_options, size_group=size_group))
 
         else:
-            section.add_row(GSettingsComboBox(_("Turn off the screen when inactive for"), CSD_SCHEMA, "sleep-display-ac", SLEEP_DELAY_OPTIONS, valtype="int", size_group=size_group))
+            section.add_row(GSettingsComboBox(_("Turn off the screen when inactive for"), CSD_SCHEMA, "sleep-display-ac", SLEEP_DELAY_OPTIONS, valtype=int, size_group=size_group))
 
-            section.add_row(GSettingsComboBox(_("Suspend when inactive for"), CSD_SCHEMA, "sleep-inactive-ac-timeout", SLEEP_DELAY_OPTIONS, valtype="int", size_group=size_group))
+            section.add_row(GSettingsComboBox(_("Suspend when inactive for"), CSD_SCHEMA, "sleep-inactive-ac-timeout", SLEEP_DELAY_OPTIONS, valtype=int, size_group=size_group))
 
             if self.has_lid:
                 section.add_row(GSettingsComboBox(_("When the lid is closed"), CSD_SCHEMA, "lid-close-ac-action", lid_options, size_group=size_group))
@@ -165,9 +169,14 @@ class Module:
         if self.has_lid:
             section.add_row(GSettingsSwitch(_("Perform lid-closed action even with external monitors attached"), CSD_SCHEMA, "lid-close-suspend-with-external-monitor"))
 
-        if self.has_battery and UPowerGlib.MAJOR_VERSION == 0 and UPowerGlib.MINOR_VERSION < 99:
+        if self.has_battery and UPowerGlib.MAJOR_VERSION == 0 and UPowerGlib.MINOR_VERSION <= 99:
             section.add_row(GSettingsComboBox(_("When the battery is critically low"), CSD_SCHEMA, "critical-battery-action", critical_options, size_group=size_group))
 
+        if can_suspend and can_hybrid_sleep:
+            switch = GSettingsSwitch(_("Enable Hybrid Sleep"), CSM_SCHEMA, "prefer-hybrid-sleep")
+            switch.set_tooltip_text(_("Replaces Suspend with Hybrid Sleep"))
+            section.add_row(switch)
+
         # Batteries
 
         self.battery_page = SettingsPage()
@@ -190,8 +199,8 @@ class Module:
                         break
                     except:
                         pass
-        except Exception, detail:
-            print "Failed to query backlight information in cs_power module: %s" % detail
+        except Exception as detail:
+            print("Failed to query backlight information in cs_power module: %s" % detail)
 
         if primary_output is None:
             if self.show_battery_page:
@@ -204,13 +213,13 @@ class Module:
             return
 
         proxy = Gio.DBusProxy.new_sync(
-                Gio.bus_get_sync(Gio.BusType.SESSION, None),
-                Gio.DBusProxyFlags.NONE,
-                None,
-                "org.cinnamon.SettingsDaemon",
-                "/org/cinnamon/SettingsDaemon/Power",
-                "org.cinnamon.SettingsDaemon.Power.Screen",
-                None)
+            Gio.bus_get_sync(Gio.BusType.SESSION, None),
+            Gio.DBusProxyFlags.NONE,
+            None,
+            "org.cinnamon.SettingsDaemon.Power",
+            "/org/cinnamon/SettingsDaemon/Power",
+            "org.cinnamon.SettingsDaemon.Power.Screen",
+            None)
 
         try:
             brightness = proxy.GetPercentage()
@@ -237,9 +246,9 @@ class Module:
 
             section.add_row(GSettingsSwitch(_("On battery, dim screen when inactive"), CSD_SCHEMA, "idle-dim-battery"))
 
-            section.add_reveal_row(GSettingsComboBox(_("Brightness level when inactive"), CSD_SCHEMA, "idle-brightness", IDLE_BRIGHTNESS_OPTIONS, valtype="int", size_group=size_group), CSD_SCHEMA, "idle-dim-battery")
+            section.add_reveal_row(GSettingsComboBox(_("Brightness level when inactive"), CSD_SCHEMA, "idle-brightness", IDLE_BRIGHTNESS_OPTIONS, valtype=int, size_group=size_group), CSD_SCHEMA, "idle-dim-battery")
 
-            section.add_reveal_row(GSettingsComboBox(_("Dim screen after inactive for"), CSD_SCHEMA, "idle-dim-time", IDLE_DELAY_OPTIONS, valtype="int", size_group=size_group), CSD_SCHEMA, "idle-dim-battery")
+            section.add_reveal_row(GSettingsComboBox(_("Dim screen after inactive for"), CSD_SCHEMA, "idle-dim-time", IDLE_DELAY_OPTIONS, valtype=int, size_group=size_group), CSD_SCHEMA, "idle-dim-battery")
 
     def build_battery_page(self, *args):
 
@@ -467,7 +476,7 @@ class Module:
         self.battery_label_size_group.add_widget(label_box)
         hbox.pack_start(label_box, False, False, 0)
         label = Gtk.Label()
-        label.set_markup("%d%%" % int(percentage))        
+        label.set_markup("%d%%" % int(percentage))
         label.set_size_request(30, -1)
         hbox.pack_start(label, False, False, 15)
 
@@ -505,20 +514,22 @@ class Module:
 def get_available_options(up_client):
     can_suspend = False
     can_hibernate = False
+    can_hybrid_sleep = False
 
     try:
         connection = Gio.bus_get_sync(Gio.BusType.SYSTEM, None)
         proxy = Gio.DBusProxy.new_sync(
-                connection,
-                Gio.DBusProxyFlags.NONE,
-                None,
-                "org.freedesktop.login1",
-                "/org/freedesktop/login1",
-                "org.freedesktop.login1.Manager",
-                None)
+            connection,
+            Gio.DBusProxyFlags.NONE,
+            None,
+            "org.freedesktop.login1",
+            "/org/freedesktop/login1",
+            "org.freedesktop.login1.Manager",
+            None)
 
         can_suspend = proxy.CanSuspend() == "yes"
         can_hibernate = proxy.CanHibernate() == "yes"
+        can_hybrid_sleep = proxy.CanHybridSleep() == "yes"
     except:
         pass
 
@@ -526,6 +537,7 @@ def get_available_options(up_client):
     try:
         can_suspend = can_suspend or up_client.get_can_suspend()
         can_hibernate = can_hibernate or up_client.get_can_hibernate()
+        can_hybrid_sleep = can_hibernate or up_client.get_can_hybrid_sleep()
     except:
         pass
 
@@ -537,6 +549,7 @@ def get_available_options(up_client):
 
     lid_options = [
         ("suspend", _("Suspend")),
+        ("shutdown", _("Shutdown immediately")),
         ("hibernate", _("Hibernate")),
         ("nothing", _("Do nothing"))
     ]
@@ -551,6 +564,7 @@ def get_available_options(up_client):
     ]
 
     critical_options = [
+        ("shutdown", _("Shutdown immediately")),
         ("hibernate", _("Hibernate")),
         ("nothing", _("Do nothing"))
     ]
@@ -563,7 +577,7 @@ def get_available_options(up_client):
         for options in lid_options, button_power_options, critical_options:
             remove(options, "hibernate")
 
-    return lid_options, button_power_options, critical_options
+    return lid_options, button_power_options, critical_options, can_suspend, can_hybrid_sleep
 
 class BrightnessSlider(SettingsWidget):
     step = 5
